searchData={"items":[{"type":"module","title":"ExLLM","doc":"ExLLM - Unified Elixir client library for Large Language Models.\n\nExLLM provides a consistent interface across multiple LLM providers including\nOpenAI, Anthropic Claude, Ollama, and others. It features configuration injection,\nstandardized error handling, and streaming support.","ref":"ExLLM.html"},{"type":"module","title":"Quick Start - ExLLM","doc":"# Using environment variables\n    messages = [%{role: \"user\", content: \"Hello!\"}]\n    {:ok, response} = ExLLM.chat(:anthropic, messages)\n    IO.puts(response.content)\n\n    # Using static configuration\n    config = %{anthropic: %{api_key: \"your-key\"}}\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)\n    {:ok, response} = ExLLM.chat(:anthropic, messages, config_provider: provider)","ref":"ExLLM.html#module-quick-start"},{"type":"module","title":"Supported Providers - ExLLM","doc":"- `:anthropic` - Anthropic Claude models\n- `:openai` - OpenAI GPT models  \n- `:ollama` - Local Ollama models\n- More providers coming soon!","ref":"ExLLM.html#module-supported-providers"},{"type":"module","title":"Features - ExLLM","doc":"- **Unified Interface**: Same API across all providers\n- **Configuration Injection**: Flexible config management\n- **Streaming Support**: Real-time response streaming\n- **Error Standardization**: Consistent error handling\n- **No Process Dependencies**: Pure functional core\n- **Extensible**: Easy to add new providers","ref":"ExLLM.html#module-features"},{"type":"module","title":"Configuration - ExLLM","doc":"ExLLM supports multiple configuration methods:","ref":"ExLLM.html#module-configuration"},{"type":"module","title":"Environment Variables - ExLLM","doc":"export OPENAI_API_KEY=\"sk-...\"\n    export ANTHROPIC_API_KEY=\"api-...\"\n    export OLLAMA_BASE_URL=\"http://localhost:11434\"","ref":"ExLLM.html#module-environment-variables"},{"type":"module","title":"Static Configuration - ExLLM","doc":"config = %{\n      openai: %{api_key: \"sk-...\", model: \"gpt-4\"},\n      anthropic: %{api_key: \"api-...\", model: \"claude-3\"},\n      ollama: %{base_url: \"http://localhost:11434\", model: \"llama2\"}\n    }\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)","ref":"ExLLM.html#module-static-configuration"},{"type":"module","title":"Custom Configuration - ExLLM","doc":"defmodule MyConfigProvider do\n      @behaviour ExLLM.ConfigProvider\n      \n      def get([:openai, :api_key]), do: MyApp.get_secret(\"openai_key\")\n      def get(_), do: nil\n      \n      def get_all(), do: %{}\n    end","ref":"ExLLM.html#module-custom-configuration"},{"type":"module","title":"Examples - ExLLM","doc":"# Simple chat\n    {:ok, response} = ExLLM.chat(:openai, [\n      %{role: \"user\", content: \"What is Elixir?\"}\n    ])\n\n    # With options\n    {:ok, response} = ExLLM.chat(:anthropic, messages,\n      model: \"claude-3-haiku-20240307\",\n      temperature: 0.7,\n      max_tokens: 1000\n    )\n\n    # Streaming\n    {:ok, stream} = ExLLM.stream_chat(:openai, messages)\n    for chunk <- stream do\n      if chunk.content, do: IO.write(chunk.content)\n    end\n\n    # Check if provider is configured\n    if ExLLM.configured?(:anthropic) do\n      {:ok, response} = ExLLM.chat(:anthropic, messages)\n    end\n\n    # List available models\n    {:ok, models} = ExLLM.list_models(:openai)\n    Enum.each(models, fn model ->\n      IO.puts(model.name)\n    end)","ref":"ExLLM.html#module-examples"},{"type":"function","title":"ExLLM.add_session_message/4","doc":"Add a message to a session.","ref":"ExLLM.html#add_session_message/4"},{"type":"function","title":"Parameters - ExLLM.add_session_message/4","doc":"- `session` - The session to update\n- `role` - Message role (\"user\", \"assistant\", etc.)\n- `content` - Message content\n- `opts` - Additional message metadata","ref":"ExLLM.html#add_session_message/4-parameters"},{"type":"function","title":"Returns - ExLLM.add_session_message/4","doc":"Updated session.","ref":"ExLLM.html#add_session_message/4-returns"},{"type":"function","title":"Examples - ExLLM.add_session_message/4","doc":"session = ExLLM.add_session_message(session, \"user\", \"What is Elixir?\")","ref":"ExLLM.html#add_session_message/4-examples"},{"type":"function","title":"ExLLM.calculate_cost/3","doc":"Calculate cost for token usage.","ref":"ExLLM.html#calculate_cost/3"},{"type":"function","title":"Parameters - ExLLM.calculate_cost/3","doc":"- `provider` - LLM provider name\n- `model` - Model name\n- `token_usage` - Map with `:input_tokens` and `:output_tokens`","ref":"ExLLM.html#calculate_cost/3-parameters"},{"type":"function","title":"Returns - ExLLM.calculate_cost/3","doc":"Cost calculation result or error map.","ref":"ExLLM.html#calculate_cost/3-returns"},{"type":"function","title":"Examples - ExLLM.calculate_cost/3","doc":"usage = %{input_tokens: 1000, output_tokens: 500}\n    cost = ExLLM.calculate_cost(\"openai\", \"gpt-4\", usage)\n    # => %{total_cost: 0.06, ...}","ref":"ExLLM.html#calculate_cost/3-examples"},{"type":"function","title":"ExLLM.chat/3","doc":"Send a chat completion request to the specified LLM provider.","ref":"ExLLM.html#chat/3"},{"type":"function","title":"Parameters - ExLLM.chat/3","doc":"- `provider` - The LLM provider (`:anthropic`, `:openai`, `:ollama`)\n- `messages` - List of conversation messages\n- `options` - Options for the request (see module docs)","ref":"ExLLM.html#chat/3-parameters"},{"type":"function","title":"Options - ExLLM.chat/3","doc":"- `:model` - Override the default model\n- `:temperature` - Temperature setting (0.0 to 1.0)\n- `:max_tokens` - Maximum tokens in response or context\n- `:config_provider` - Configuration provider module or pid\n- `:track_cost` - Whether to track costs (default: true)\n- `:strategy` - Context truncation strategy (default: :sliding_window)\n  - `:sliding_window` - Keep most recent messages\n  - `:smart` - Preserve system messages and recent context\n- `:preserve_messages` - Number of recent messages to always preserve (default: 5)\n- `:response_model` - Ecto schema or type spec for structured output (requires instructor)\n- `:max_retries` - Number of retries for structured output validation","ref":"ExLLM.html#chat/3-options"},{"type":"function","title":"Returns - ExLLM.chat/3","doc":"`{:ok, %ExLLM.Types.LLMResponse{}}` on success, or `{:ok, struct}` when using\nresponse_model. Returns `{:error, reason}` on failure.","ref":"ExLLM.html#chat/3-returns"},{"type":"function","title":"Examples - ExLLM.chat/3","doc":"# Simple usage\n    {:ok, response} = ExLLM.chat(:anthropic, [\n      %{role: \"user\", content: \"Hello!\"}\n    ])\n\n    # With custom configuration\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(%{\n      anthropic: %{api_key: \"your-key\"}\n    })\n    {:ok, response} = ExLLM.chat(:anthropic, messages, config_provider: provider)\n\n    # With model override\n    {:ok, response} = ExLLM.chat(:openai, messages, model: \"gpt-4-turbo\")\n\n    # With context management\n    {:ok, response} = ExLLM.chat(:anthropic, messages,\n      max_tokens: 4000,\n      strategy: :smart\n    )\n\n    # With structured output (requires instructor)\n    {:ok, classification} = ExLLM.chat(:anthropic, messages,\n      response_model: EmailClassification,\n      max_retries: 3\n    )","ref":"ExLLM.html#chat/3-examples"},{"type":"function","title":"ExLLM.chat_with_session/3","doc":"Send a chat request using a session, automatically tracking messages and usage.","ref":"ExLLM.html#chat_with_session/3"},{"type":"function","title":"Parameters - ExLLM.chat_with_session/3","doc":"- `session` - The session to use\n- `content` - The user message content\n- `options` - Chat options (same as `chat/3`)","ref":"ExLLM.html#chat_with_session/3-parameters"},{"type":"function","title":"Returns - ExLLM.chat_with_session/3","doc":"`{:ok, {response, updated_session}}` on success, `{:error, reason}` on failure.","ref":"ExLLM.html#chat_with_session/3-returns"},{"type":"function","title":"Examples - ExLLM.chat_with_session/3","doc":"session = ExLLM.new_session(:anthropic)\n    {:ok, {response, session}} = ExLLM.chat_with_session(session, \"Hello!\")\n    # Session now contains the conversation history","ref":"ExLLM.html#chat_with_session/3-examples"},{"type":"function","title":"ExLLM.clear_session/1","doc":"Clear messages from a session while preserving metadata.","ref":"ExLLM.html#clear_session/1"},{"type":"function","title":"Parameters - ExLLM.clear_session/1","doc":"- `session` - The session to clear","ref":"ExLLM.html#clear_session/1-parameters"},{"type":"function","title":"Returns - ExLLM.clear_session/1","doc":"Updated session with no messages.","ref":"ExLLM.html#clear_session/1-returns"},{"type":"function","title":"Examples - ExLLM.clear_session/1","doc":"session = ExLLM.clear_session(session)","ref":"ExLLM.html#clear_session/1-examples"},{"type":"function","title":"ExLLM.configured?/2","doc":"Check if the specified provider is properly configured.","ref":"ExLLM.html#configured?/2"},{"type":"function","title":"Parameters - ExLLM.configured?/2","doc":"- `provider` - The LLM provider to check\n- `options` - Options including configuration provider","ref":"ExLLM.html#configured?/2-parameters"},{"type":"function","title":"Returns - ExLLM.configured?/2","doc":"`true` if configured, `false` otherwise.","ref":"ExLLM.html#configured?/2-returns"},{"type":"function","title":"Examples - ExLLM.configured?/2","doc":"if ExLLM.configured?(:anthropic) do\n      {:ok, response} = ExLLM.chat(:anthropic, messages)\n    else\n      IO.puts(\"Anthropic not configured\")\n    end","ref":"ExLLM.html#configured?/2-examples"},{"type":"function","title":"ExLLM.context_stats/1","doc":"Get statistics about message context usage.","ref":"ExLLM.html#context_stats/1"},{"type":"function","title":"Parameters - ExLLM.context_stats/1","doc":"- `messages` - List of conversation messages","ref":"ExLLM.html#context_stats/1-parameters"},{"type":"function","title":"Returns - ExLLM.context_stats/1","doc":"Map with context statistics.","ref":"ExLLM.html#context_stats/1-returns"},{"type":"function","title":"Examples - ExLLM.context_stats/1","doc":"stats = ExLLM.context_stats(messages)\n    # => %{total_tokens: 1500, message_count: 10, ...}","ref":"ExLLM.html#context_stats/1-examples"},{"type":"function","title":"ExLLM.context_window_size/2","doc":"Get context window size for a model.","ref":"ExLLM.html#context_window_size/2"},{"type":"function","title":"Parameters - ExLLM.context_window_size/2","doc":"- `provider` - LLM provider name\n- `model` - Model name","ref":"ExLLM.html#context_window_size/2-parameters"},{"type":"function","title":"Returns - ExLLM.context_window_size/2","doc":"Context window size in tokens or nil if unknown.","ref":"ExLLM.html#context_window_size/2-returns"},{"type":"function","title":"Examples - ExLLM.context_window_size/2","doc":"tokens = ExLLM.context_window_size(:anthropic, \"claude-3-5-sonnet-20241022\")\n    # => 200000","ref":"ExLLM.html#context_window_size/2-examples"},{"type":"function","title":"ExLLM.default_model/1","doc":"Get the default model for the specified provider.","ref":"ExLLM.html#default_model/1"},{"type":"function","title":"Parameters - ExLLM.default_model/1","doc":"- `provider` - The LLM provider","ref":"ExLLM.html#default_model/1-parameters"},{"type":"function","title":"Returns - ExLLM.default_model/1","doc":"String model identifier.","ref":"ExLLM.html#default_model/1-returns"},{"type":"function","title":"Examples - ExLLM.default_model/1","doc":"model = ExLLM.default_model(:anthropic)\n    # => \"claude-sonnet-4-20250514\"","ref":"ExLLM.html#default_model/1-examples"},{"type":"function","title":"ExLLM.estimate_tokens/1","doc":"Estimate token count for text.","ref":"ExLLM.html#estimate_tokens/1"},{"type":"function","title":"Parameters - ExLLM.estimate_tokens/1","doc":"- `text` - Text to analyze (string, message map, or list)","ref":"ExLLM.html#estimate_tokens/1-parameters"},{"type":"function","title":"Returns - ExLLM.estimate_tokens/1","doc":"Estimated token count.","ref":"ExLLM.html#estimate_tokens/1-returns"},{"type":"function","title":"Examples - ExLLM.estimate_tokens/1","doc":"tokens = ExLLM.estimate_tokens(\"Hello, world!\")\n    # => 4","ref":"ExLLM.html#estimate_tokens/1-examples"},{"type":"function","title":"ExLLM.format_cost/1","doc":"Format cost for display.","ref":"ExLLM.html#format_cost/1"},{"type":"function","title":"Parameters - ExLLM.format_cost/1","doc":"- `cost` - Cost in dollars","ref":"ExLLM.html#format_cost/1-parameters"},{"type":"function","title":"Returns - ExLLM.format_cost/1","doc":"Formatted cost string.","ref":"ExLLM.html#format_cost/1-returns"},{"type":"function","title":"Examples - ExLLM.format_cost/1","doc":"ExLLM.format_cost(0.0035)\n    # => \"$0.350¢\"","ref":"ExLLM.html#format_cost/1-examples"},{"type":"function","title":"ExLLM.get_session_messages/2","doc":"Get messages from a session.","ref":"ExLLM.html#get_session_messages/2"},{"type":"function","title":"Parameters - ExLLM.get_session_messages/2","doc":"- `session` - The session to query\n- `limit` - Optional message limit","ref":"ExLLM.html#get_session_messages/2-parameters"},{"type":"function","title":"Returns - ExLLM.get_session_messages/2","doc":"List of messages.","ref":"ExLLM.html#get_session_messages/2-returns"},{"type":"function","title":"Examples - ExLLM.get_session_messages/2","doc":"messages = ExLLM.get_session_messages(session)\n    last_10 = ExLLM.get_session_messages(session, 10)","ref":"ExLLM.html#get_session_messages/2-examples"},{"type":"function","title":"ExLLM.list_models/2","doc":"List available models for the specified provider.","ref":"ExLLM.html#list_models/2"},{"type":"function","title":"Parameters - ExLLM.list_models/2","doc":"- `provider` - The LLM provider\n- `options` - Options including configuration provider","ref":"ExLLM.html#list_models/2-parameters"},{"type":"function","title":"Returns - ExLLM.list_models/2","doc":"`{:ok, [%ExLLM.Types.Model{}]}` on success, `{:error, reason}` on failure.","ref":"ExLLM.html#list_models/2-returns"},{"type":"function","title":"Examples - ExLLM.list_models/2","doc":"{:ok, models} = ExLLM.list_models(:anthropic)\n    Enum.each(models, fn model ->\n      IO.puts(model.name)\n    end)","ref":"ExLLM.html#list_models/2-examples"},{"type":"function","title":"ExLLM.load_session/1","doc":"Load a session from JSON.","ref":"ExLLM.html#load_session/1"},{"type":"function","title":"Parameters - ExLLM.load_session/1","doc":"- `json` - JSON string containing session data","ref":"ExLLM.html#load_session/1-parameters"},{"type":"function","title":"Returns - ExLLM.load_session/1","doc":"`{:ok, session}` on success, `{:error, reason}` on failure.","ref":"ExLLM.html#load_session/1-returns"},{"type":"function","title":"Examples - ExLLM.load_session/1","doc":"json = File.read!(\"session.json\")\n    {:ok, session} = ExLLM.load_session(json)","ref":"ExLLM.html#load_session/1-examples"},{"type":"type","title":"ExLLM.messages/0","doc":"","ref":"ExLLM.html#t:messages/0"},{"type":"function","title":"ExLLM.new_session/2","doc":"Create a new conversation session.","ref":"ExLLM.html#new_session/2"},{"type":"function","title":"Parameters - ExLLM.new_session/2","doc":"- `provider` - LLM provider to use for the session\n- `opts` - Session options (`:name` for session name)","ref":"ExLLM.html#new_session/2-parameters"},{"type":"function","title":"Returns - ExLLM.new_session/2","doc":"A new session struct.","ref":"ExLLM.html#new_session/2-returns"},{"type":"function","title":"Examples - ExLLM.new_session/2","doc":"session = ExLLM.new_session(:anthropic)\n    session = ExLLM.new_session(:openai, name: \"Customer Support\")","ref":"ExLLM.html#new_session/2-examples"},{"type":"type","title":"ExLLM.options/0","doc":"","ref":"ExLLM.html#t:options/0"},{"type":"function","title":"ExLLM.prepare_messages/2","doc":"Prepare messages for sending to a provider with context management.","ref":"ExLLM.html#prepare_messages/2"},{"type":"function","title":"Parameters - ExLLM.prepare_messages/2","doc":"- `messages` - List of conversation messages\n- `options` - Options for context management","ref":"ExLLM.html#prepare_messages/2-parameters"},{"type":"function","title":"Options - ExLLM.prepare_messages/2","doc":"- `:max_tokens` - Maximum tokens for context (default: model-specific)\n- `:strategy` - Context truncation strategy (default: :sliding_window)\n- `:preserve_messages` - Number of recent messages to preserve (default: 5)","ref":"ExLLM.html#prepare_messages/2-options"},{"type":"function","title":"Returns - ExLLM.prepare_messages/2","doc":"Prepared messages list that fits within context window.","ref":"ExLLM.html#prepare_messages/2-returns"},{"type":"function","title":"Examples - ExLLM.prepare_messages/2","doc":"messages = ExLLM.prepare_messages(long_conversation,\n      max_tokens: 4000,\n      strategy: :smart\n    )","ref":"ExLLM.html#prepare_messages/2-examples"},{"type":"type","title":"ExLLM.provider/0","doc":"","ref":"ExLLM.html#t:provider/0"},{"type":"function","title":"ExLLM.save_session/1","doc":"Save a session to JSON.","ref":"ExLLM.html#save_session/1"},{"type":"function","title":"Parameters - ExLLM.save_session/1","doc":"- `session` - The session to save","ref":"ExLLM.html#save_session/1-parameters"},{"type":"function","title":"Returns - ExLLM.save_session/1","doc":"`{:ok, json}` on success, `{:error, reason}` on failure.","ref":"ExLLM.html#save_session/1-returns"},{"type":"function","title":"Examples - ExLLM.save_session/1","doc":"{:ok, json} = ExLLM.save_session(session)\n    File.write!(\"session.json\", json)","ref":"ExLLM.html#save_session/1-examples"},{"type":"function","title":"ExLLM.session_token_usage/1","doc":"Get total token usage for a session.","ref":"ExLLM.html#session_token_usage/1"},{"type":"function","title":"Parameters - ExLLM.session_token_usage/1","doc":"- `session` - The session to analyze","ref":"ExLLM.html#session_token_usage/1-parameters"},{"type":"function","title":"Returns - ExLLM.session_token_usage/1","doc":"Total token count.","ref":"ExLLM.html#session_token_usage/1-returns"},{"type":"function","title":"Examples - ExLLM.session_token_usage/1","doc":"tokens = ExLLM.session_token_usage(session)\n    # => 2500","ref":"ExLLM.html#session_token_usage/1-examples"},{"type":"function","title":"ExLLM.stream_chat/3","doc":"Send a streaming chat completion request to the specified LLM provider.","ref":"ExLLM.html#stream_chat/3"},{"type":"function","title":"Parameters - ExLLM.stream_chat/3","doc":"- `provider` - The LLM provider (`:anthropic`, `:openai`, `:ollama`)\n- `messages` - List of conversation messages\n- `options` - Options for the request (see module docs)","ref":"ExLLM.html#stream_chat/3-parameters"},{"type":"function","title":"Options - ExLLM.stream_chat/3","doc":"Same as `chat/3`, plus:\n- `:on_chunk` - Callback function for each chunk","ref":"ExLLM.html#stream_chat/3-options"},{"type":"function","title":"Returns - ExLLM.stream_chat/3","doc":"`{:ok, stream}` on success where stream yields `%ExLLM.Types.StreamChunk{}` structs,\n`{:error, reason}` on failure.","ref":"ExLLM.html#stream_chat/3-returns"},{"type":"function","title":"Examples - ExLLM.stream_chat/3","doc":"{:ok, stream} = ExLLM.stream_chat(:anthropic, messages)\n    \n    # Process the stream\n    for chunk <- stream do\n      case chunk do\n        %{content: content} when content != nil ->\n          IO.write(content)\n        %{finish_reason: \"stop\"} ->\n          IO.puts(\"\\nDone!\")\n        _ ->\n          :continue\n      end\n    end\n\n    # With context management\n    {:ok, stream} = ExLLM.stream_chat(:anthropic, messages,\n      max_tokens: 4000,\n      strategy: :smart\n    )","ref":"ExLLM.html#stream_chat/3-examples"},{"type":"function","title":"ExLLM.supported_providers/0","doc":"Get list of supported providers.","ref":"ExLLM.html#supported_providers/0"},{"type":"function","title":"Returns - ExLLM.supported_providers/0","doc":"List of provider atoms.","ref":"ExLLM.html#supported_providers/0-returns"},{"type":"function","title":"Examples - ExLLM.supported_providers/0","doc":"providers = ExLLM.supported_providers()\n    # => [:anthropic, :openai, :ollama]","ref":"ExLLM.html#supported_providers/0-examples"},{"type":"function","title":"ExLLM.validate_context/2","doc":"Validate that messages fit within a model's context window.","ref":"ExLLM.html#validate_context/2"},{"type":"function","title":"Parameters - ExLLM.validate_context/2","doc":"- `messages` - List of conversation messages\n- `options` - Options including model info","ref":"ExLLM.html#validate_context/2-parameters"},{"type":"function","title":"Returns - ExLLM.validate_context/2","doc":"`{:ok, token_count}` if valid, `{:error, reason}` if too large.","ref":"ExLLM.html#validate_context/2-returns"},{"type":"function","title":"Examples - ExLLM.validate_context/2","doc":"{:ok, tokens} = ExLLM.validate_context(messages, model: \"claude-3-5-sonnet-20241022\")\n    # => {:ok, 3500}","ref":"ExLLM.html#validate_context/2-examples"},{"type":"behaviour","title":"ExLLM.Adapter","doc":"Behaviour for LLM backend adapters.\n\nThis defines the common interface that all LLM adapters must implement,\nenabling a unified API across different providers.","ref":"ExLLM.Adapter.html"},{"type":"behaviour","title":"Example Implementation - ExLLM.Adapter","doc":"defmodule MyAdapter do\n      @behaviour ExLLM.Adapter\n      alias ExLLM.Types\n      \n      @impl true\n      def chat(messages, options \\ []) do\n        # Implementation here\n        {:ok, %Types.LLMResponse{content: \"Hello\"}}\n      end\n      \n      @impl true\n      def stream_chat(messages, options \\ []) do\n        # Implementation here\n        {:ok, stream}\n      end\n      \n      @impl true\n      def configured?(options \\ []) do\n        # Check if adapter is configured\n        true\n      end\n      \n      @impl true\n      def default_model() do\n        \"my-model\"\n      end\n      \n      @impl true\n      def list_models(options \\ []) do\n        {:ok, [\"model1\", \"model2\"]}\n      end\n    end","ref":"ExLLM.Adapter.html#module-example-implementation"},{"type":"behaviour","title":"Options - ExLLM.Adapter","doc":"All adapter functions accept an options keyword list that can include:\n\n- `:config_provider` - Configuration provider for API keys and settings\n- `:model` - Override the default model for this request\n- `:temperature` - Control randomness (0.0 to 1.0)\n- `:max_tokens` - Maximum tokens in response\n- `:timeout` - Request timeout in milliseconds","ref":"ExLLM.Adapter.html#module-options"},{"type":"behaviour","title":"Configuration Injection - ExLLM.Adapter","doc":"Adapters support configuration injection through the `:config_provider` option:\n\n    # Use environment variables\n    ExLLM.OpenAI.chat(messages, config_provider: ExLLM.ConfigProvider.Env)\n    \n    # Use static config\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(%{\n      openai: %{api_key: \"sk-...\"}\n    })\n    ExLLM.OpenAI.chat(messages, config_provider: provider)","ref":"ExLLM.Adapter.html#module-configuration-injection"},{"type":"callback","title":"ExLLM.Adapter.chat/2","doc":"Send a chat completion request to the LLM.","ref":"ExLLM.Adapter.html#c:chat/2"},{"type":"callback","title":"Parameters - ExLLM.Adapter.chat/2","doc":"- `messages` - List of conversation messages\n- `options` - Adapter options (see module documentation)","ref":"ExLLM.Adapter.html#c:chat/2-parameters"},{"type":"callback","title":"Returns - ExLLM.Adapter.chat/2","doc":"`{:ok, %ExLLM.Types.LLMResponse{}}` on success, `{:error, reason}` on failure.","ref":"ExLLM.Adapter.html#c:chat/2-returns"},{"type":"callback","title":"ExLLM.Adapter.configured?/1","doc":"Check if the adapter is properly configured and ready to use.","ref":"ExLLM.Adapter.html#c:configured?/1"},{"type":"callback","title":"Parameters - ExLLM.Adapter.configured?/1","doc":"- `options` - Adapter options (see module documentation)","ref":"ExLLM.Adapter.html#c:configured?/1-parameters"},{"type":"callback","title":"Returns - ExLLM.Adapter.configured?/1","doc":"`true` if configured, `false` otherwise.","ref":"ExLLM.Adapter.html#c:configured?/1-returns"},{"type":"callback","title":"ExLLM.Adapter.default_model/0","doc":"Get the default model for this adapter.","ref":"ExLLM.Adapter.html#c:default_model/0"},{"type":"callback","title":"Returns - ExLLM.Adapter.default_model/0","doc":"String model identifier.","ref":"ExLLM.Adapter.html#c:default_model/0-returns"},{"type":"callback","title":"ExLLM.Adapter.list_models/1","doc":"List available models for this adapter.","ref":"ExLLM.Adapter.html#c:list_models/1"},{"type":"callback","title":"Parameters - ExLLM.Adapter.list_models/1","doc":"- `options` - Adapter options (see module documentation)","ref":"ExLLM.Adapter.html#c:list_models/1-parameters"},{"type":"callback","title":"Returns - ExLLM.Adapter.list_models/1","doc":"`{:ok, [%ExLLM.Types.Model{}]}` on success, `{:error, reason}` on failure.","ref":"ExLLM.Adapter.html#c:list_models/1-returns"},{"type":"callback","title":"ExLLM.Adapter.stream_chat/2","doc":"Send a streaming chat completion request to the LLM.","ref":"ExLLM.Adapter.html#c:stream_chat/2"},{"type":"callback","title":"Parameters - ExLLM.Adapter.stream_chat/2","doc":"- `messages` - List of conversation messages  \n- `options` - Adapter options (see module documentation)","ref":"ExLLM.Adapter.html#c:stream_chat/2-parameters"},{"type":"callback","title":"Returns - ExLLM.Adapter.stream_chat/2","doc":"`{:ok, stream}` on success where stream yields `%ExLLM.Types.StreamChunk{}` structs,\n`{:error, reason}` on failure.","ref":"ExLLM.Adapter.html#c:stream_chat/2-returns"},{"type":"module","title":"ExLLM.Adapters.Anthropic","doc":"Anthropic Claude API adapter for ExLLM.","ref":"ExLLM.Adapters.Anthropic.html"},{"type":"module","title":"Configuration - ExLLM.Adapters.Anthropic","doc":"This adapter requires an Anthropic API key and optionally a base URL.","ref":"ExLLM.Adapters.Anthropic.html#module-configuration"},{"type":"module","title":"Using Environment Variables - ExLLM.Adapters.Anthropic","doc":"# Set environment variables\n    export ANTHROPIC_API_KEY=\"your-api-key\"\n    export ANTHROPIC_MODEL=\"claude-3-5-sonnet-20241022\"  # optional\n\n    # Use with default environment provider\n    ExLLM.Adapters.Anthropic.chat(messages, config_provider: ExLLM.ConfigProvider.Env)","ref":"ExLLM.Adapters.Anthropic.html#module-using-environment-variables"},{"type":"module","title":"Using Static Configuration - ExLLM.Adapters.Anthropic","doc":"config = %{\n      anthropic: %{\n        api_key: \"your-api-key\",\n        model: \"claude-3-5-sonnet-20241022\",\n        base_url: \"https://api.anthropic.com/v1\"  # optional\n      }\n    }\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)\n    ExLLM.Adapters.Anthropic.chat(messages, config_provider: provider)","ref":"ExLLM.Adapters.Anthropic.html#module-using-static-configuration"},{"type":"module","title":"Example Usage - ExLLM.Adapters.Anthropic","doc":"messages = [\n      %{role: \"user\", content: \"Hello, how are you?\"}\n    ]\n\n    # Simple chat\n    {:ok, response} = ExLLM.Adapters.Anthropic.chat(messages)\n    IO.puts(response.content)\n\n    # Streaming chat\n    {:ok, stream} = ExLLM.Adapters.Anthropic.stream_chat(messages)\n    for chunk <- stream do\n      if chunk.content, do: IO.write(chunk.content)\n    end","ref":"ExLLM.Adapters.Anthropic.html#module-example-usage"},{"type":"module","title":"ExLLM.Adapters.Local","doc":"Local LLM adapter using Bumblebee for on-device inference.\n\nThis adapter enables running language models locally using Bumblebee and EXLA/EMLX backends.\nIt supports Apple Silicon (via EMLX), NVIDIA GPUs (via CUDA), and CPU inference.","ref":"ExLLM.Adapters.Local.html"},{"type":"module","title":"Configuration - ExLLM.Adapters.Local","doc":"The local adapter doesn't require API keys but may need backend configuration:\n\n    # For Apple Silicon (automatic detection)\n    {:ok, response} = ExLLM.chat(:local, messages)\n    \n    # With specific model\n    {:ok, response} = ExLLM.chat(:local, messages, model: \"microsoft/phi-2\")","ref":"ExLLM.Adapters.Local.html#module-configuration"},{"type":"module","title":"Available Models - ExLLM.Adapters.Local","doc":"- `microsoft/phi-2` - Phi-2 (2.7B) - Default\n- `meta-llama/Llama-2-7b-hf` - Llama 2 (7B)\n- `mistralai/Mistral-7B-v0.1` - Mistral (7B)\n- `EleutherAI/gpt-neo-1.3B` - GPT-Neo (1.3B)\n- `google/flan-t5-base` - Flan-T5 Base","ref":"ExLLM.Adapters.Local.html#module-available-models"},{"type":"module","title":"Features - ExLLM.Adapters.Local","doc":"- On-device inference with no API calls\n- Automatic hardware acceleration detection\n- Support for Apple Silicon, NVIDIA GPUs, and CPUs\n- Model caching for faster subsequent loads\n- Streaming support for real-time generation","ref":"ExLLM.Adapters.Local.html#module-features"},{"type":"behaviour","title":"ExLLM.ConfigProvider","doc":"Behaviour for configuration providers.\n\nThis allows modules to receive configuration through dependency injection\nrather than directly accessing application configuration, making them more \nportable and testable.","ref":"ExLLM.ConfigProvider.html"},{"type":"behaviour","title":"Example - ExLLM.ConfigProvider","doc":"# Using static configuration\n    config = %{\n      openai: %{api_key: \"sk-...\"},\n      anthropic: %{api_key: \"api-...\"}\n    }\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)\n    ExLLM.OpenAI.chat(messages, config_provider: provider)\n\n    # Using environment-based configuration\n    ExLLM.OpenAI.chat(messages, config_provider: ExLLM.ConfigProvider.Env)","ref":"ExLLM.ConfigProvider.html#module-example"},{"type":"callback","title":"ExLLM.ConfigProvider.get/2","doc":"Gets configuration value for a provider and key.","ref":"ExLLM.ConfigProvider.html#c:get/2"},{"type":"callback","title":"ExLLM.ConfigProvider.get_all/1","doc":"Gets all configuration for a provider.","ref":"ExLLM.ConfigProvider.html#c:get_all/1"},{"type":"module","title":"ExLLM.ConfigProvider.Env","doc":"Environment-based configuration provider.\n\nReads configuration from environment variables using standard naming:\n- `OPENAI_API_KEY` for OpenAI\n- `ANTHROPIC_API_KEY` for Anthropic\n- etc.","ref":"ExLLM.ConfigProvider.Env.html"},{"type":"module","title":"ExLLM.ConfigProvider.Static","doc":"Static configuration provider for testing and library usage.","ref":"ExLLM.ConfigProvider.Static.html"},{"type":"module","title":"Usage - ExLLM.ConfigProvider.Static","doc":"config = %{\n      openai: %{api_key: \"sk-test\", model: \"gpt-4\"},\n      anthropic: %{api_key: \"api-test\", model: \"claude-3\"}\n    }\n    {:ok, provider} = ExLLM.ConfigProvider.Static.start_link(config)\n    ExLLM.OpenAI.chat(messages, config_provider: provider)","ref":"ExLLM.ConfigProvider.Static.html#module-usage"},{"type":"function","title":"ExLLM.ConfigProvider.Static.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExLLM.ConfigProvider.Static.html#child_spec/1"},{"type":"function","title":"ExLLM.ConfigProvider.Static.start_link/1","doc":"","ref":"ExLLM.ConfigProvider.Static.html#start_link/1"},{"type":"module","title":"ExLLM.Context","doc":"Context management for LLM conversations.\n\nProvides utilities for managing conversation context windows, including:\n- Token counting and estimation\n- Message truncation strategies\n- Context window validation\n- System prompt preservation","ref":"ExLLM.Context.html"},{"type":"function","title":"ExLLM.Context.context_window_size/2","doc":"Get context window size for a specific model.","ref":"ExLLM.Context.html#context_window_size/2"},{"type":"function","title":"Parameters - ExLLM.Context.context_window_size/2","doc":"- `provider` - LLM provider name (string)\n- `model` - Model name","ref":"ExLLM.Context.html#context_window_size/2-parameters"},{"type":"function","title":"Returns - ExLLM.Context.context_window_size/2","doc":"Context window size in tokens or nil if unknown.","ref":"ExLLM.Context.html#context_window_size/2-returns"},{"type":"function","title":"ExLLM.Context.find_models_by_context/1","doc":"Find models with context windows above a certain size.","ref":"ExLLM.Context.html#find_models_by_context/1"},{"type":"function","title":"Examples - ExLLM.Context.find_models_by_context/1","doc":"# Find models with at least 100k token context\n    large_models = ExLLM.Context.find_models_by_context(min_tokens: 100_000)","ref":"ExLLM.Context.html#find_models_by_context/1-examples"},{"type":"function","title":"ExLLM.Context.fits_context?/4","doc":"Check if messages fit within context window.","ref":"ExLLM.Context.html#fits_context?/4"},{"type":"function","title":"Parameters - ExLLM.Context.fits_context?/4","doc":"- `provider` - LLM provider\n- `model` - Model name\n- `messages` - Messages to check","ref":"ExLLM.Context.html#fits_context?/4-parameters"},{"type":"function","title":"Returns - ExLLM.Context.fits_context?/4","doc":"`{:ok, true}` if fits, `{:ok, false}` if not, `{:error, reason}` if unknown model.","ref":"ExLLM.Context.html#fits_context?/4-returns"},{"type":"function","title":"Examples - ExLLM.Context.fits_context?/4","doc":"{:ok, fits} = ExLLM.Context.fits_context?(:anthropic, \"claude-3-5-sonnet-20241022\", messages)","ref":"ExLLM.Context.html#fits_context?/4-examples"},{"type":"function","title":"ExLLM.Context.get_context_window/2","doc":"Get context window size for a model.","ref":"ExLLM.Context.html#get_context_window/2"},{"type":"function","title":"Examples - ExLLM.Context.get_context_window/2","doc":"window = ExLLM.Context.get_context_window(:openai, \"gpt-4\")\n    # => 8192","ref":"ExLLM.Context.html#get_context_window/2-examples"},{"type":"function","title":"ExLLM.Context.get_stats/2","doc":"Get context statistics for messages.","ref":"ExLLM.Context.html#get_stats/2"},{"type":"function","title":"Returns - ExLLM.Context.get_stats/2","doc":"Map with message count, token usage, and percentages.","ref":"ExLLM.Context.html#get_stats/2-returns"},{"type":"function","title":"Examples - ExLLM.Context.get_stats/2","doc":"stats = ExLLM.Context.get_stats(messages, max_tokens: 4096)\n    # => %{\n    #   message_count: 10,\n    #   estimated_tokens: 1234,\n    #   max_tokens: 4096,\n    #   tokens_used_percentage: 30.1,\n    #   tokens_remaining: 2362\n    # }","ref":"ExLLM.Context.html#get_stats/2-examples"},{"type":"function","title":"ExLLM.Context.list_model_contexts/0","doc":"List all models with their context windows.","ref":"ExLLM.Context.html#list_model_contexts/0"},{"type":"function","title":"Examples - ExLLM.Context.list_model_contexts/0","doc":"models = ExLLM.Context.list_model_contexts()\n    # => [\n    #   %{provider: \"anthropic\", model: \"claude-3-5-sonnet-20241022\", context_window: 200000},\n    #   ...\n    # ]","ref":"ExLLM.Context.html#list_model_contexts/0-examples"},{"type":"function","title":"ExLLM.Context.prepare_messages/2","doc":"Prepare messages for sending to LLM, managing context window.","ref":"ExLLM.Context.html#prepare_messages/2"},{"type":"function","title":"Options - ExLLM.Context.prepare_messages/2","doc":"- `:max_tokens` - Maximum context window size (defaults to model's limit)\n- `:system_prompt` - System prompt to prepend\n- `:strategy` - Truncation strategy (`:sliding_window`, `:smart`, `:none`)\n- `:preserve_system` - Always keep system messages (default: true)\n- `:reserve_tokens` - Tokens to reserve for response (default: 500)","ref":"ExLLM.Context.html#prepare_messages/2-options"},{"type":"function","title":"Examples - ExLLM.Context.prepare_messages/2","doc":"messages = [\n      %{role: \"system\", content: \"You are helpful.\"},\n      %{role: \"user\", content: \"Hello\"},\n      %{role: \"assistant\", content: \"Hi!\"}\n    ]\n\n    # Auto-detect context window for model\n    prepared = ExLLM.Context.prepare_messages(messages, \n      provider: :anthropic,\n      model: \"claude-3-5-sonnet-20241022\"\n    )\n\n    # Manual context limit\n    prepared = ExLLM.Context.prepare_messages(messages,\n      max_tokens: 4096,\n      strategy: :smart\n    )","ref":"ExLLM.Context.html#prepare_messages/2-examples"},{"type":"function","title":"ExLLM.Context.stats/1","doc":"Get statistics about message context usage.","ref":"ExLLM.Context.html#stats/1"},{"type":"function","title":"Parameters - ExLLM.Context.stats/1","doc":"- `messages` - List of conversation messages","ref":"ExLLM.Context.html#stats/1-parameters"},{"type":"function","title":"Returns - ExLLM.Context.stats/1","doc":"Map with detailed statistics.","ref":"ExLLM.Context.html#stats/1-returns"},{"type":"function","title":"ExLLM.Context.truncate_messages/2","doc":"Truncate messages to fit within token limit.","ref":"ExLLM.Context.html#truncate_messages/2"},{"type":"function","title":"Parameters - ExLLM.Context.truncate_messages/2","doc":"- `messages` - List of messages\n- `options` - Options including `:max_tokens`","ref":"ExLLM.Context.html#truncate_messages/2-parameters"},{"type":"function","title":"Returns - ExLLM.Context.truncate_messages/2","doc":"Truncated list of messages.","ref":"ExLLM.Context.html#truncate_messages/2-returns"},{"type":"function","title":"ExLLM.Context.validate_context/2","doc":"Validate that messages fit within a model's context window.","ref":"ExLLM.Context.html#validate_context/2"},{"type":"function","title":"Options - ExLLM.Context.validate_context/2","doc":"- `:provider` - LLM provider name\n- `:model` - Model name\n- `:max_tokens` - Override default context window","ref":"ExLLM.Context.html#validate_context/2-options"},{"type":"function","title":"Returns - ExLLM.Context.validate_context/2","doc":"`{:ok, token_count}` if valid, `{:error, {:context_too_large, details}}` if too large.","ref":"ExLLM.Context.html#validate_context/2-returns"},{"type":"module","title":"ExLLM.Cost","doc":"Cost calculation functionality for ExLLM.\n\nProvides token estimation, cost calculation, and pricing information\nfor all supported LLM providers.","ref":"ExLLM.Cost.html"},{"type":"function","title":"ExLLM.Cost.calculate/3","doc":"Calculate cost for token usage.","ref":"ExLLM.Cost.html#calculate/3"},{"type":"function","title":"ExLLM.Cost.compare/2","doc":"Compare costs across different providers for the same usage.","ref":"ExLLM.Cost.html#compare/2"},{"type":"function","title":"ExLLM.Cost.estimate_tokens/1","doc":"Estimate token count for text using heuristic analysis.","ref":"ExLLM.Cost.html#estimate_tokens/1"},{"type":"function","title":"ExLLM.Cost.format/1","doc":"Format cost for human-readable display.","ref":"ExLLM.Cost.html#format/1"},{"type":"function","title":"ExLLM.Cost.get_pricing/2","doc":"Get pricing for a specific provider and model.","ref":"ExLLM.Cost.html#get_pricing/2"},{"type":"function","title":"ExLLM.Cost.list_pricing/0","doc":"List all available models and their pricing.","ref":"ExLLM.Cost.html#list_pricing/0"},{"type":"module","title":"ExLLM.Error","doc":"Standardized error types and utilities for ExLLM.\n\nThis module defines consistent error patterns used throughout ExLLM\nto provide predictable error handling for library consumers.","ref":"ExLLM.Error.html"},{"type":"module","title":"Error Patterns - ExLLM.Error","doc":"All functions should return either `{:ok, result}` or `{:error, reason}`.\n\nError reasons follow these patterns:","ref":"ExLLM.Error.html#module-error-patterns"},{"type":"module","title":"Simple Errors (atoms) - ExLLM.Error","doc":"- `:not_found` - Resource not found\n- `:not_connected` - Connection not established\n- `:invalid_config` - Configuration is invalid\n- `:not_configured` - Required configuration missing\n- `:timeout` - Operation timed out\n- `:unauthorized` - Authentication failed","ref":"ExLLM.Error.html#module-simple-errors-atoms"},{"type":"module","title":"Complex Errors (tuples) - ExLLM.Error","doc":"- `{:api_error, details}` - API call failed\n- `{:validation, field, message}` - Validation failed\n- `{:connection_failed, reason}` - Connection failure\n- `{:json_parse_error, reason}` - JSON parsing failed","ref":"ExLLM.Error.html#module-complex-errors-tuples"},{"type":"module","title":"Examples - ExLLM.Error","doc":"# Simple error\n    {:error, :not_found}\n\n    # API error with details\n    {:error, {:api_error, %{status: 404, body: \"Not found\"}}}\n\n    # Validation error\n    {:error, {:validation, :name, \"cannot be empty\"}}\n\n    # Connection error\n    {:error, {:connection_failed, :timeout}}","ref":"ExLLM.Error.html#module-examples"},{"type":"function","title":"ExLLM.Error.api_error/2","doc":"Creates a standardized API error.","ref":"ExLLM.Error.html#api_error/2"},{"type":"function","title":"Parameters - ExLLM.Error.api_error/2","doc":"- `status` - HTTP status code\n- `body` - Response body (string or map)","ref":"ExLLM.Error.html#api_error/2-parameters"},{"type":"function","title":"Returns - ExLLM.Error.api_error/2","doc":"API error tuple","ref":"ExLLM.Error.html#api_error/2-returns"},{"type":"type","title":"ExLLM.Error.complex_error/0","doc":"","ref":"ExLLM.Error.html#t:complex_error/0"},{"type":"function","title":"ExLLM.Error.connection_error/1","doc":"Creates a standardized connection failure error.","ref":"ExLLM.Error.html#connection_error/1"},{"type":"function","title":"Parameters - ExLLM.Error.connection_error/1","doc":"- `reason` - Underlying failure reason","ref":"ExLLM.Error.html#connection_error/1-parameters"},{"type":"function","title":"Returns - ExLLM.Error.connection_error/1","doc":"Connection error tuple","ref":"ExLLM.Error.html#connection_error/1-returns"},{"type":"function","title":"ExLLM.Error.error?/1","doc":"Checks if a value is an error tuple.","ref":"ExLLM.Error.html#error?/1"},{"type":"function","title":"Parameters - ExLLM.Error.error?/1","doc":"- `value` - Value to check","ref":"ExLLM.Error.html#error?/1-parameters"},{"type":"function","title":"Returns - ExLLM.Error.error?/1","doc":"Boolean indicating if value is an error tuple","ref":"ExLLM.Error.html#error?/1-returns"},{"type":"type","title":"ExLLM.Error.error_reason/0","doc":"","ref":"ExLLM.Error.html#t:error_reason/0"},{"type":"function","title":"ExLLM.Error.get_error_reason/1","doc":"Extracts error reason from error tuple.","ref":"ExLLM.Error.html#get_error_reason/1"},{"type":"function","title":"Parameters - ExLLM.Error.get_error_reason/1","doc":"- `error_tuple` - Error tuple","ref":"ExLLM.Error.html#get_error_reason/1-parameters"},{"type":"function","title":"Returns - ExLLM.Error.get_error_reason/1","doc":"Error reason or nil if not an error tuple","ref":"ExLLM.Error.html#get_error_reason/1-returns"},{"type":"function","title":"ExLLM.Error.json_parse_error/1","doc":"Creates a standardized JSON parsing error.","ref":"ExLLM.Error.html#json_parse_error/1"},{"type":"function","title":"Parameters - ExLLM.Error.json_parse_error/1","doc":"- `reason` - JSON parsing failure reason","ref":"ExLLM.Error.html#json_parse_error/1-parameters"},{"type":"function","title":"Returns - ExLLM.Error.json_parse_error/1","doc":"JSON parse error tuple","ref":"ExLLM.Error.html#json_parse_error/1-returns"},{"type":"type","title":"ExLLM.Error.result/1","doc":"","ref":"ExLLM.Error.html#t:result/1"},{"type":"type","title":"ExLLM.Error.simple_error/0","doc":"","ref":"ExLLM.Error.html#t:simple_error/0"},{"type":"function","title":"ExLLM.Error.validation_error/2","doc":"Creates a standardized validation error.","ref":"ExLLM.Error.html#validation_error/2"},{"type":"function","title":"Parameters - ExLLM.Error.validation_error/2","doc":"- `field` - Field name (atom)\n- `message` - Error message (string)","ref":"ExLLM.Error.html#validation_error/2-parameters"},{"type":"function","title":"Returns - ExLLM.Error.validation_error/2","doc":"Validation error tuple","ref":"ExLLM.Error.html#validation_error/2-returns"},{"type":"module","title":"ExLLM.Instructor","doc":"Structured output support for ExLLM using instructor_ex.\n\nThis module provides integration with the `instructor` library to enable\nstructured outputs with validation when using ExLLM. It allows you to\ndefine expected response structures using Ecto schemas and automatically\nvalidates and retries LLM responses.","ref":"ExLLM.Instructor.html"},{"type":"module","title":"Requirements - ExLLM.Instructor","doc":"This module requires the optional `instructor` dependency:\n\n    {:instructor, \"~> 0.1.0\"}","ref":"ExLLM.Instructor.html#module-requirements"},{"type":"module","title":"Usage - ExLLM.Instructor","doc":"","ref":"ExLLM.Instructor.html#module-usage"},{"type":"module","title":"Basic Example - ExLLM.Instructor","doc":"defmodule EmailClassification do\n      use Ecto.Schema\n      use Instructor.Validator\n\n      @llm_doc \"Classification of an email as spam or not spam\"\n      \n      @primary_key false\n      embedded_schema do\n        field :classification, Ecto.Enum, values: [:spam, :not_spam]\n        field :confidence, :float\n        field :reason, :string\n      end\n\n      @impl true\n      def validate_changeset(changeset) do\n        changeset\n        |> Ecto.Changeset.validate_required([:classification, :confidence, :reason])\n        |> Ecto.Changeset.validate_number(:confidence, \n            greater_than_or_equal_to: 0.0,\n            less_than_or_equal_to: 1.0\n          )\n      end\n    end\n\n    # Use with ExLLM\n    {:ok, result} = ExLLM.Instructor.chat(:anthropic, [\n      %{role: \"user\", content: \"Is this spam? 'You won a million dollars!'\"}\n    ], response_model: EmailClassification)\n\n    # result is now an EmailClassification struct\n    # %EmailClassification{classification: :spam, confidence: 0.95, reason: \"...\"}","ref":"ExLLM.Instructor.html#module-basic-example"},{"type":"module","title":"With Retries - ExLLM.Instructor","doc":"{:ok, result} = ExLLM.Instructor.chat(:anthropic, messages,\n      response_model: UserProfile,\n      max_retries: 3,\n      temperature: 0.7\n    )","ref":"ExLLM.Instructor.html#module-with-retries"},{"type":"module","title":"With Simple Maps - ExLLM.Instructor","doc":"# Define expected structure\n    response_model = %{\n      name: :string,\n      age: :integer,\n      tags: {:array, :string}\n    }\n\n    {:ok, result} = ExLLM.Instructor.chat(:anthropic, messages,\n      response_model: response_model\n    )","ref":"ExLLM.Instructor.html#module-with-simple-maps"},{"type":"module","title":"Integration with ExLLM - ExLLM.Instructor","doc":"The structured output functionality is also available through the main\nExLLM module when instructor is installed:\n\n    {:ok, response} = ExLLM.chat(:anthropic, messages,\n      response_model: EmailClassification,\n      max_retries: 2\n    )","ref":"ExLLM.Instructor.html#module-integration-with-exllm"},{"type":"function","title":"ExLLM.Instructor.available?/0","doc":"Check if instructor is available.","ref":"ExLLM.Instructor.html#available?/0"},{"type":"function","title":"Returns - ExLLM.Instructor.available?/0","doc":"`true` if the instructor library is available, `false` otherwise.","ref":"ExLLM.Instructor.html#available?/0-returns"},{"type":"function","title":"Examples - ExLLM.Instructor.available?/0","doc":"if ExLLM.Instructor.available?() do\n      # Use structured outputs\n    else\n      # Fall back to regular chat\n    end","ref":"ExLLM.Instructor.html#available?/0-examples"},{"type":"function","title":"ExLLM.Instructor.chat/3","doc":"Send a chat request with structured output validation.","ref":"ExLLM.Instructor.html#chat/3"},{"type":"function","title":"Parameters - ExLLM.Instructor.chat/3","doc":"- `provider` - The LLM provider to use\n- `messages` - List of conversation messages\n- `options` - Options including `:response_model` and standard chat options","ref":"ExLLM.Instructor.html#chat/3-parameters"},{"type":"function","title":"Options - ExLLM.Instructor.chat/3","doc":"- `:response_model` - Required. Ecto schema module or simple type specification\n- `:max_retries` - Number of retries for validation errors (default: 0)\n- All standard ExLLM.chat/3 options","ref":"ExLLM.Instructor.html#chat/3-options"},{"type":"function","title":"Returns - ExLLM.Instructor.chat/3","doc":"- `{:ok, struct}` where struct matches the response_model\n- `{:error, reason}` on failure","ref":"ExLLM.Instructor.html#chat/3-returns"},{"type":"function","title":"Examples - ExLLM.Instructor.chat/3","doc":"# With Ecto schema\n    {:ok, classification} = ExLLM.Instructor.chat(:anthropic, messages,\n      response_model: EmailClassification,\n      max_retries: 3\n    )\n\n    # With simple type spec\n    {:ok, data} = ExLLM.Instructor.chat(:openai, messages,\n      response_model: %{name: :string, age: :integer}\n    )","ref":"ExLLM.Instructor.html#chat/3-examples"},{"type":"function","title":"ExLLM.Instructor.parse_response/2","doc":"Transform a regular ExLLM response into a structured output.\n\nThis function is useful when you already have a response from ExLLM\nand want to parse it into a structured format.","ref":"ExLLM.Instructor.html#parse_response/2"},{"type":"function","title":"Parameters - ExLLM.Instructor.parse_response/2","doc":"- `response` - An ExLLM.Types.LLMResponse struct\n- `response_model` - The expected structure (Ecto schema or type spec)","ref":"ExLLM.Instructor.html#parse_response/2-parameters"},{"type":"function","title":"Returns - ExLLM.Instructor.parse_response/2","doc":"- `{:ok, struct}` on successful parsing and validation\n- `{:error, reason}` on failure","ref":"ExLLM.Instructor.html#parse_response/2-returns"},{"type":"function","title":"Examples - ExLLM.Instructor.parse_response/2","doc":"{:ok, response} = ExLLM.chat(:anthropic, messages)\n    {:ok, structured} = ExLLM.Instructor.parse_response(response, UserProfile)","ref":"ExLLM.Instructor.html#parse_response/2-examples"},{"type":"function","title":"ExLLM.Instructor.simple_schema/2","doc":"Create a simple schema module at runtime.\n\nThis is a convenience function for creating simple schemas without\ndefining a full module.","ref":"ExLLM.Instructor.html#simple_schema/2"},{"type":"function","title":"Parameters - ExLLM.Instructor.simple_schema/2","doc":"- `fields` - Map of field names to types\n- `validations` - Optional list of validation functions","ref":"ExLLM.Instructor.html#simple_schema/2-parameters"},{"type":"function","title":"Examples - ExLLM.Instructor.simple_schema/2","doc":"schema = ExLLM.Instructor.simple_schema(%{\n      name: :string,\n      age: :integer,\n      email: :string\n    }, [\n      {:validate_format, :email, ~r/@/}\n    ])\n\n    {:ok, result} = ExLLM.Instructor.chat(:anthropic, messages,\n      response_model: schema\n    )","ref":"ExLLM.Instructor.html#simple_schema/2-examples"},{"type":"module","title":"ExLLM.Local.EXLAConfig","doc":"Configuration module for EXLA/EMLX backend optimization.\n\nProvides optimal settings for CPU and GPU inference, including Apple Silicon support.\nThis module automatically detects available hardware acceleration and configures\nthe appropriate backend for best performance.","ref":"ExLLM.Local.EXLAConfig.html"},{"type":"module","title":"Supported Backends - ExLLM.Local.EXLAConfig","doc":"- **EMLX** - Apple Silicon (Metal) acceleration\n- **CUDA** - NVIDIA GPU acceleration\n- **ROCm** - AMD GPU acceleration  \n- **CPU** - Optimized CPU inference","ref":"ExLLM.Local.EXLAConfig.html#module-supported-backends"},{"type":"module","title":"Features - ExLLM.Local.EXLAConfig","doc":"- Automatic hardware detection\n- Mixed precision support\n- Memory optimization\n- Dynamic batch sizing\n- Parallel execution configuration","ref":"ExLLM.Local.EXLAConfig.html#module-features"},{"type":"function","title":"ExLLM.Local.EXLAConfig.acceleration_info/0","doc":"Get information about available acceleration.\n\nReturns a map with acceleration details including type, name, and capabilities.","ref":"ExLLM.Local.EXLAConfig.html#acceleration_info/0"},{"type":"function","title":"ExLLM.Local.EXLAConfig.configure_backend/0","doc":"Configure EXLA/EMLX backend with optimal settings based on available hardware.\n\nReturns `{:ok, backend}` where backend is `:emlx`, `:cuda`, `:rocm`, `:cpu`, or `:binary`.","ref":"ExLLM.Local.EXLAConfig.html#configure_backend/0"},{"type":"function","title":"ExLLM.Local.EXLAConfig.determine_backend_options/0","doc":"Determine optimal backend options based on available hardware.\n\nReturns a map of backend configuration options.","ref":"ExLLM.Local.EXLAConfig.html#determine_backend_options/0"},{"type":"function","title":"ExLLM.Local.EXLAConfig.enable_mixed_precision/0","doc":"Enable mixed precision training/inference for better performance.","ref":"ExLLM.Local.EXLAConfig.html#enable_mixed_precision/0"},{"type":"function","title":"ExLLM.Local.EXLAConfig.optimize_memory/0","doc":"Optimize memory usage for large models.","ref":"ExLLM.Local.EXLAConfig.html#optimize_memory/0"},{"type":"function","title":"ExLLM.Local.EXLAConfig.serving_options/0","doc":"Get optimal compiler options for model serving.\n\nReturns keyword list of options for Bumblebee serving configuration.","ref":"ExLLM.Local.EXLAConfig.html#serving_options/0"},{"type":"module","title":"ExLLM.Local.ModelLoader","doc":"Handles loading and caching of Bumblebee models for local inference.\n\nThis GenServer manages the lifecycle of loaded models, ensuring efficient\nmemory usage and providing fast access to cached models.","ref":"ExLLM.Local.ModelLoader.html"},{"type":"module","title":"Features - ExLLM.Local.ModelLoader","doc":"- Automatic model downloading from HuggingFace\n- Model caching to avoid reloading\n- Memory management with model unloading\n- Hardware acceleration detection\n- Support for local model paths","ref":"ExLLM.Local.ModelLoader.html#module-features"},{"type":"function","title":"ExLLM.Local.ModelLoader.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExLLM.Local.ModelLoader.html#child_spec/1"},{"type":"function","title":"ExLLM.Local.ModelLoader.get_acceleration_info/0","doc":"Get information about hardware acceleration.","ref":"ExLLM.Local.ModelLoader.html#get_acceleration_info/0"},{"type":"function","title":"ExLLM.Local.ModelLoader.get_model_info/1","doc":"Get information about a loaded model.","ref":"ExLLM.Local.ModelLoader.html#get_model_info/1"},{"type":"function","title":"ExLLM.Local.ModelLoader.init/1","doc":"Callback implementation for  GenServer.init/1 .","ref":"ExLLM.Local.ModelLoader.html#init/1"},{"type":"function","title":"ExLLM.Local.ModelLoader.list_loaded_models/0","doc":"List all loaded models.","ref":"ExLLM.Local.ModelLoader.html#list_loaded_models/0"},{"type":"function","title":"ExLLM.Local.ModelLoader.load_model/1","doc":"Load a model by name or path. Returns {:ok, model_info} or {:error, reason}.","ref":"ExLLM.Local.ModelLoader.html#load_model/1"},{"type":"function","title":"Examples - ExLLM.Local.ModelLoader.load_model/1","doc":"{:ok, model} = ModelLoader.load_model(\"microsoft/phi-2\")\n    {:ok, model} = ModelLoader.load_model(\"/path/to/local/model\")","ref":"ExLLM.Local.ModelLoader.html#load_model/1-examples"},{"type":"function","title":"ExLLM.Local.ModelLoader.start_link/1","doc":"","ref":"ExLLM.Local.ModelLoader.html#start_link/1"},{"type":"function","title":"ExLLM.Local.ModelLoader.unload_model/1","doc":"Unload a model from memory.","ref":"ExLLM.Local.ModelLoader.html#unload_model/1"},{"type":"module","title":"ExLLM.Local.TokenCounter","doc":"Token counting utilities for local models.\n\nProvides accurate token counting when Bumblebee is available,\nwith fallback to heuristic estimation.","ref":"ExLLM.Local.TokenCounter.html"},{"type":"function","title":"ExLLM.Local.TokenCounter.count_messages/2","doc":"Count tokens for a list of messages.","ref":"ExLLM.Local.TokenCounter.html#count_messages/2"},{"type":"function","title":"Examples - ExLLM.Local.TokenCounter.count_messages/2","doc":"messages = [\n      %{role: \"user\", content: \"Hello\"},\n      %{role: \"assistant\", content: \"Hi there!\"}\n    ]\n    {:ok, count} = TokenCounter.count_messages(messages, model: \"microsoft/phi-2\")","ref":"ExLLM.Local.TokenCounter.html#count_messages/2-examples"},{"type":"function","title":"ExLLM.Local.TokenCounter.count_tokens/2","doc":"Count tokens for the given text using the specified model's tokenizer.\n\nFalls back to heuristic estimation if model is not loaded or Bumblebee\nis not available.","ref":"ExLLM.Local.TokenCounter.html#count_tokens/2"},{"type":"function","title":"Examples - ExLLM.Local.TokenCounter.count_tokens/2","doc":"{:ok, count} = TokenCounter.count_tokens(\"Hello world\", model: \"microsoft/phi-2\")\n    # => {:ok, 2}\n    \n    # Fallback when model not loaded\n    {:ok, estimate} = TokenCounter.count_tokens(\"Hello world\")\n    # => {:ok, 3}","ref":"ExLLM.Local.TokenCounter.html#count_tokens/2-examples"},{"type":"module","title":"ExLLM.Session","doc":"Session management for ExLLM - handles conversation sessions with LLM providers.\n\nExLLM.Session provides pure functional operations for managing conversation sessions,\nincluding message history, token tracking, and session persistence. All operations\nare stateless and return updated session structs.","ref":"ExLLM.Session.html"},{"type":"module","title":"Quick Start - ExLLM.Session","doc":"# Create a new session\n    session = ExLLM.Session.new(\"anthropic\")\n\n    # Add messages\n    session = ExLLM.Session.add_message(session, \"user\", \"Hello!\")\n    session = ExLLM.Session.add_message(session, \"assistant\", \"Hi there!\")\n\n    # Get messages\n    messages = ExLLM.Session.get_messages(session)\n\n    # Update token usage\n    session = ExLLM.Session.update_token_usage(session, %{input_tokens: 10, output_tokens: 15})","ref":"ExLLM.Session.html#module-quick-start"},{"type":"module","title":"Features - ExLLM.Session","doc":"- **Pure Functional**: All operations are stateless and immutable\n- **Message Management**: Add, retrieve, and filter conversation messages\n- **Token Tracking**: Track input/output token usage across conversations\n- **Context Storage**: Store arbitrary metadata with sessions\n- **Session Persistence**: Serialize/deserialize sessions to/from JSON\n- **LLM Integration**: Seamlessly works with ExLLM providers","ref":"ExLLM.Session.html#module-features"},{"type":"module","title":"Session Structure - ExLLM.Session","doc":"Sessions are represented by `ExLLM.Session.Types.Session` structs containing:\n- `id` - Unique session identifier\n- `llm_backend` - LLM backend name (optional)\n- `messages` - List of conversation messages\n- `context` - Arbitrary metadata map\n- `created_at`/`updated_at` - Timestamps\n- `token_usage` - Token consumption tracking\n- `name` - Human-readable session name (optional)","ref":"ExLLM.Session.html#module-session-structure"},{"type":"function","title":"ExLLM.Session.add_message/4","doc":"Add a message to the session.","ref":"ExLLM.Session.html#add_message/4"},{"type":"function","title":"Parameters - ExLLM.Session.add_message/4","doc":"- `session` - The session to update\n- `role` - Message role (\"user\", \"assistant\", etc.)\n- `content` - Message content\n- `opts` - Additional message metadata","ref":"ExLLM.Session.html#add_message/4-parameters"},{"type":"function","title":"Returns - ExLLM.Session.add_message/4","doc":"Updated session with the new message.","ref":"ExLLM.Session.html#add_message/4-returns"},{"type":"function","title":"Examples - ExLLM.Session.add_message/4","doc":"session = ExLLM.Session.add_message(session, \"user\", \"Hello!\")\n    session = ExLLM.Session.add_message(session, \"assistant\", \"Hi there!\", timestamp: DateTime.utc_now())","ref":"ExLLM.Session.html#add_message/4-examples"},{"type":"function","title":"ExLLM.Session.clear_messages/1","doc":"Clear all messages from the session.","ref":"ExLLM.Session.html#clear_messages/1"},{"type":"function","title":"Parameters - ExLLM.Session.clear_messages/1","doc":"- `session` - The session to clear","ref":"ExLLM.Session.html#clear_messages/1-parameters"},{"type":"function","title":"Returns - ExLLM.Session.clear_messages/1","doc":"Updated session with empty message list.","ref":"ExLLM.Session.html#clear_messages/1-returns"},{"type":"function","title":"Examples - ExLLM.Session.clear_messages/1","doc":"session = ExLLM.Session.clear_messages(session)","ref":"ExLLM.Session.html#clear_messages/1-examples"},{"type":"function","title":"ExLLM.Session.from_json/1","doc":"Deserialize session from JSON.","ref":"ExLLM.Session.html#from_json/1"},{"type":"function","title":"Parameters - ExLLM.Session.from_json/1","doc":"- `json_string` - JSON string to deserialize","ref":"ExLLM.Session.html#from_json/1-parameters"},{"type":"function","title":"Returns - ExLLM.Session.from_json/1","doc":"`{:ok, session}` on success, `{:error, reason}` on failure.","ref":"ExLLM.Session.html#from_json/1-returns"},{"type":"function","title":"Examples - ExLLM.Session.from_json/1","doc":"{:ok, session} = ExLLM.Session.from_json(json_string)","ref":"ExLLM.Session.html#from_json/1-examples"},{"type":"function","title":"ExLLM.Session.get_messages/2","doc":"Get messages from the session, optionally with a limit.","ref":"ExLLM.Session.html#get_messages/2"},{"type":"function","title":"Parameters - ExLLM.Session.get_messages/2","doc":"- `session` - The session to get messages from\n- `limit` - Maximum number of messages to return (optional)","ref":"ExLLM.Session.html#get_messages/2-parameters"},{"type":"function","title":"Returns - ExLLM.Session.get_messages/2","doc":"List of messages, most recent first if limited.","ref":"ExLLM.Session.html#get_messages/2-returns"},{"type":"function","title":"Examples - ExLLM.Session.get_messages/2","doc":"all_messages = ExLLM.Session.get_messages(session)\n    last_5 = ExLLM.Session.get_messages(session, 5)","ref":"ExLLM.Session.html#get_messages/2-examples"},{"type":"function","title":"ExLLM.Session.new/2","doc":"Create a new session with the specified backend.","ref":"ExLLM.Session.html#new/2"},{"type":"function","title":"Parameters - ExLLM.Session.new/2","doc":"- `backend` - LLM backend to use (optional)\n- `opts` - Additional options (`:name` for session name)","ref":"ExLLM.Session.html#new/2-parameters"},{"type":"function","title":"Returns - ExLLM.Session.new/2","doc":"A new Session struct.","ref":"ExLLM.Session.html#new/2-returns"},{"type":"function","title":"Examples - ExLLM.Session.new/2","doc":"session = ExLLM.Session.new(\"anthropic\")\n    session = ExLLM.Session.new(\"openai\", name: \"My Chat\")","ref":"ExLLM.Session.html#new/2-examples"},{"type":"function","title":"ExLLM.Session.set_context/2","doc":"Set context for the session.","ref":"ExLLM.Session.html#set_context/2"},{"type":"function","title":"Parameters - ExLLM.Session.set_context/2","doc":"- `session` - The session to update\n- `context` - Context map to set","ref":"ExLLM.Session.html#set_context/2-parameters"},{"type":"function","title":"Returns - ExLLM.Session.set_context/2","doc":"Updated session with new context.","ref":"ExLLM.Session.html#set_context/2-returns"},{"type":"function","title":"Examples - ExLLM.Session.set_context/2","doc":"session = ExLLM.Session.set_context(session, %{temperature: 0.7, max_tokens: 1000})","ref":"ExLLM.Session.html#set_context/2-examples"},{"type":"function","title":"ExLLM.Session.set_name/2","doc":"Set the name of the session.","ref":"ExLLM.Session.html#set_name/2"},{"type":"function","title":"Parameters - ExLLM.Session.set_name/2","doc":"- `session` - The session to update\n- `name` - New session name","ref":"ExLLM.Session.html#set_name/2-parameters"},{"type":"function","title":"Returns - ExLLM.Session.set_name/2","doc":"Updated session with new name.","ref":"ExLLM.Session.html#set_name/2-returns"},{"type":"function","title":"Examples - ExLLM.Session.set_name/2","doc":"session = ExLLM.Session.set_name(session, \"My Important Chat\")","ref":"ExLLM.Session.html#set_name/2-examples"},{"type":"function","title":"ExLLM.Session.to_json/1","doc":"Serialize session to JSON.","ref":"ExLLM.Session.html#to_json/1"},{"type":"function","title":"Parameters - ExLLM.Session.to_json/1","doc":"- `session` - The session to serialize","ref":"ExLLM.Session.html#to_json/1-parameters"},{"type":"function","title":"Returns - ExLLM.Session.to_json/1","doc":"`{:ok, json_string}` on success, `{:error, reason}` on failure.","ref":"ExLLM.Session.html#to_json/1-returns"},{"type":"function","title":"Examples - ExLLM.Session.to_json/1","doc":"{:ok, json} = ExLLM.Session.to_json(session)","ref":"ExLLM.Session.html#to_json/1-examples"},{"type":"function","title":"ExLLM.Session.total_tokens/1","doc":"Get the total token count for the session.","ref":"ExLLM.Session.html#total_tokens/1"},{"type":"function","title":"Parameters - ExLLM.Session.total_tokens/1","doc":"- `session` - The session to analyze","ref":"ExLLM.Session.html#total_tokens/1-parameters"},{"type":"function","title":"Returns - ExLLM.Session.total_tokens/1","doc":"Total token count (input + output).","ref":"ExLLM.Session.html#total_tokens/1-returns"},{"type":"function","title":"Examples - ExLLM.Session.total_tokens/1","doc":"total = ExLLM.Session.total_tokens(session)","ref":"ExLLM.Session.html#total_tokens/1-examples"},{"type":"function","title":"ExLLM.Session.update_token_usage/2","doc":"Update the token usage for the session.","ref":"ExLLM.Session.html#update_token_usage/2"},{"type":"function","title":"Parameters - ExLLM.Session.update_token_usage/2","doc":"- `session` - The session to update\n- `usage` - Token usage map with `:input_tokens` and `:output_tokens`","ref":"ExLLM.Session.html#update_token_usage/2-parameters"},{"type":"function","title":"Returns - ExLLM.Session.update_token_usage/2","doc":"Updated session with new token usage.","ref":"ExLLM.Session.html#update_token_usage/2-returns"},{"type":"function","title":"Examples - ExLLM.Session.update_token_usage/2","doc":"session = ExLLM.Session.update_token_usage(session, %{input_tokens: 100, output_tokens: 150})","ref":"ExLLM.Session.html#update_token_usage/2-examples"},{"type":"module","title":"ExLLM.Session.Types","doc":"Type definitions for ExLLM.Session.","ref":"ExLLM.Session.Types.html"},{"type":"type","title":"ExLLM.Session.Types.context/0","doc":"","ref":"ExLLM.Session.Types.html#t:context/0"},{"type":"type","title":"ExLLM.Session.Types.message/0","doc":"","ref":"ExLLM.Session.Types.html#t:message/0"},{"type":"type","title":"ExLLM.Session.Types.token_usage/0","doc":"","ref":"ExLLM.Session.Types.html#t:token_usage/0"},{"type":"module","title":"ExLLM.Session.Types.Session","doc":"Represents a conversation session with message history and metadata.","ref":"ExLLM.Session.Types.Session.html"},{"type":"type","title":"ExLLM.Session.Types.Session.t/0","doc":"","ref":"ExLLM.Session.Types.Session.html#t:t/0"},{"type":"module","title":"ExLLM.Types","doc":"Shared type definitions used across ExLLM modules.\n\nThis module contains struct definitions and types that are used by multiple\nmodules, helping to avoid circular dependencies.","ref":"ExLLM.Types.html"},{"type":"type","title":"ExLLM.Types.adapter_options/0","doc":"","ref":"ExLLM.Types.html#t:adapter_options/0"},{"type":"type","title":"ExLLM.Types.cost_result/0","doc":"","ref":"ExLLM.Types.html#t:cost_result/0"},{"type":"type","title":"ExLLM.Types.message/0","doc":"","ref":"ExLLM.Types.html#t:message/0"},{"type":"type","title":"ExLLM.Types.pricing/0","doc":"","ref":"ExLLM.Types.html#t:pricing/0"},{"type":"type","title":"ExLLM.Types.stream/0","doc":"","ref":"ExLLM.Types.html#t:stream/0"},{"type":"type","title":"ExLLM.Types.token_usage/0","doc":"","ref":"ExLLM.Types.html#t:token_usage/0"},{"type":"module","title":"ExLLM.Types.LLMResponse","doc":"Standard response format from LLM adapters with integrated cost calculation.","ref":"ExLLM.Types.LLMResponse.html"},{"type":"type","title":"ExLLM.Types.LLMResponse.t/0","doc":"","ref":"ExLLM.Types.LLMResponse.html#t:t/0"},{"type":"module","title":"ExLLM.Types.Model","doc":"Represents an available LLM model.","ref":"ExLLM.Types.Model.html"},{"type":"type","title":"ExLLM.Types.Model.t/0","doc":"","ref":"ExLLM.Types.Model.html#t:t/0"},{"type":"module","title":"ExLLM.Types.StreamChunk","doc":"Represents a chunk from a streaming LLM response.","ref":"ExLLM.Types.StreamChunk.html"},{"type":"type","title":"ExLLM.Types.StreamChunk.t/0","doc":"","ref":"ExLLM.Types.StreamChunk.html#t:t/0"},{"type":"extras","title":"ExLLM","doc":"# ExLLM\n\nA unified Elixir client for Large Language Models with integrated cost tracking, providing a consistent interface across multiple LLM providers.","ref":"readme.html"},{"type":"extras","title":"Features - ExLLM","doc":"- **Unified API**: Single interface for multiple LLM providers\n- **Streaming Support**: Real-time streaming responses via Server-Sent Events\n- **Cost Tracking**: Automatic cost calculation for all API calls\n- **Token Estimation**: Heuristic-based token counting for cost prediction\n- **Context Management**: Automatic message truncation to fit model context windows\n- **Session Management**: Built-in conversation state tracking and persistence\n- **Structured Outputs**: Schema validation and retries via instructor_ex integration\n- **Configurable**: Flexible configuration system with multiple providers\n- **Type Safety**: Comprehensive typespecs and structured data\n- **Error Handling**: Consistent error patterns across all providers\n- **Extensible**: Easy to add new LLM providers via adapter pattern","ref":"readme.html#features"},{"type":"extras","title":"Supported Providers - ExLLM","doc":"- **Anthropic Claude** (claude-3-5-sonnet-20241022, claude-3-5-haiku-20241022, etc.)\n- **Local Models** (Phi-2, Llama 2, Mistral, GPT-Neo, Flan-T5 via Bumblebee)\n- **OpenAI** (coming soon)\n- **Ollama** (coming soon)","ref":"readme.html#supported-providers"},{"type":"extras","title":"Installation - ExLLM","doc":"Add `ex_llm` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_llm, \"~> 0.1.0\"},\n    \n    # Optional: For local model support\n    {:bumblebee, \"~> 0.5\", optional: true},\n    {:nx, \"~> 0.7\", optional: true},\n    {:exla, \"~> 0.7\", optional: true}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start - ExLLM","doc":"","ref":"readme.html#quick-start"},{"type":"extras","title":"Configuration - ExLLM","doc":"Configure your LLM providers in `config/config.exs`:\n\n```elixir\nconfig :ex_llm,\n  anthropic: [\n    api_key: System.get_env(\"ANTHROPIC_API_KEY\"),\n    base_url: \"https://api.anthropic.com\"\n  ]\n```","ref":"readme.html#configuration"},{"type":"extras","title":"Basic Usage - ExLLM","doc":"```elixir\n# Simple chat completion with automatic cost tracking\nmessages = [\n  %{role: \"user\", content: \"Hello, how are you?\"}\n]\n\n{:ok, response} = ExLLM.chat(:anthropic, messages)\nIO.puts(response.content)\nIO.puts(\"Cost: #{ExLLM.format_cost(response.cost.total_cost)}\")\n\n# Using local models (no API costs!)\n{:ok, response} = ExLLM.chat(:local, messages, model: \"microsoft/phi-2\")\nIO.puts(response.content)\n\n# Streaming chat\nExLLM.stream_chat(:anthropic, messages, fn chunk ->\n  IO.write(chunk.content)\nend)\n\n# Estimate tokens before making a request\ntokens = ExLLM.estimate_tokens(messages)\nIO.puts(\"Estimated tokens: #{tokens}\")\n\n# Calculate cost for specific usage\nusage = %{input_tokens: 1000, output_tokens: 500}\ncost = ExLLM.calculate_cost(:openai, \"gpt-4\", usage)\nIO.puts(\"Total cost: #{ExLLM.format_cost(cost.total_cost)}\")\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"Advanced Usage - ExLLM","doc":"```elixir\n# With custom options\noptions = [\n  model: \"claude-3-5-sonnet-20241022\",\n  max_tokens: 1000,\n  temperature: 0.7\n]\n\n{:ok, response} = ExLLM.chat(:anthropic, messages, options)\n\n# Check provider configuration\ncase ExLLM.configured?(:anthropic) do\n  true -> IO.puts(\"Anthropic is ready!\")\n  false -> IO.puts(\"Please configure Anthropic API key\")\nend\n\n# List available models\n{:ok, models} = ExLLM.list_models(:anthropic)\nEnum.each(models, &IO.puts(&1.name))\n\n# Context management - automatically truncate long conversations\nlong_conversation = [\n  %{role: \"system\", content: \"You are a helpful assistant.\"},\n  # ... many messages ...\n  %{role: \"user\", content: \"What's the weather?\"}\n]\n\n# Automatically truncates to fit model's context window\n{:ok, response} = ExLLM.chat(:anthropic, long_conversation,\n  max_tokens: 4000,        # Max tokens for context\n  strategy: :smart         # Preserve system messages and recent context\n)\n```","ref":"readme.html#advanced-usage"},{"type":"extras","title":"Session Management - ExLLM","doc":"```elixir\n# Create a new conversation session\nsession = ExLLM.new_session(:anthropic, name: \"Customer Support\")\n\n# Chat with automatic session tracking\n{:ok, {response, session}} = ExLLM.chat_with_session(session, \"Hello!\")\nIO.puts(response.content)\n\n# Continue the conversation\n{:ok, {response, session}} = ExLLM.chat_with_session(session, \"What can you help me with?\")\n\n# Session automatically tracks:\n# - Message history\n# - Token usage\n# - Conversation context\n\n# Review session details\nmessages = ExLLM.get_session_messages(session)\ntotal_tokens = ExLLM.session_token_usage(session)\nIO.puts(\"Total tokens used: #{total_tokens}\")\n\n# Save session for later\n{:ok, json} = ExLLM.save_session(session)\nFile.write!(\"session.json\", json)\n\n# Load session later\n{:ok, session} = ExLLM.load_session(File.read!(\"session.json\"))\n```","ref":"readme.html#session-management"},{"type":"extras","title":"API Reference - ExLLM","doc":"","ref":"readme.html#api-reference"},{"type":"extras","title":"Core Functions - ExLLM","doc":"- `chat/3` - Send messages and get a complete response\n- `stream_chat/3` - Send messages and stream the response\n- `configured?/2` - Check if a provider is properly configured\n- `list_models/2` - Get available models for a provider\n- `prepare_messages/2` - Prepare messages for context window\n- `validate_context/2` - Validate messages fit within context window\n- `context_window_size/2` - Get context window size for a model\n- `context_stats/1` - Get statistics about message context usage","ref":"readme.html#core-functions"},{"type":"extras","title":"Session Functions - ExLLM","doc":"- `new_session/2` - Create a new conversation session\n- `chat_with_session/3` - Chat with automatic session tracking\n- `add_session_message/4` - Add a message to a session\n- `get_session_messages/2` - Retrieve messages from a session\n- `session_token_usage/1` - Get total token usage for a session\n- `clear_session/1` - Clear messages while preserving metadata\n- `save_session/1` - Serialize session to JSON\n- `load_session/1` - Load session from JSON","ref":"readme.html#session-functions"},{"type":"extras","title":"Data Structures - ExLLM","doc":"#### LLMResponse\n\n```elixir\n%ExLLM.Types.LLMResponse{\n  content: \"Hello! I'm doing well, thank you for asking.\",\n  usage: %{input_tokens: 12, output_tokens: 15},\n  model: \"claude-3-5-sonnet-20241022\",\n  finish_reason: \"end_turn\",\n  cost: %{\n    total_cost: 0.000261,\n    input_cost: 0.000036,\n    output_cost: 0.000225,\n    currency: \"USD\"\n  }\n}\n```\n\n#### StreamChunk\n\n```elixir\n%ExLLM.Types.StreamChunk{\n  content: \"Hello\",\n  delta: true,\n  finish_reason: nil\n}\n```\n\n#### Model\n\n```elixir\n%ExLLM.Types.Model{\n  name: \"claude-3-5-sonnet-20241022\",\n  provider: :anthropic,\n  context_length: 200000,\n  supports_streaming: true\n}\n```","ref":"readme.html#data-structures"},{"type":"extras","title":"Cost Tracking - ExLLM","doc":"ExLLM automatically tracks costs for all API calls when usage data is available:","ref":"readme.html#cost-tracking"},{"type":"extras","title":"Automatic Cost Calculation - ExLLM","doc":"```elixir\n{:ok, response} = ExLLM.chat(:anthropic, messages)\n\n# Access cost information\nif response.cost do\n  IO.puts(\"Input tokens: #{response.cost.input_tokens}\")\n  IO.puts(\"Output tokens: #{response.cost.output_tokens}\") \n  IO.puts(\"Total cost: #{ExLLM.format_cost(response.cost.total_cost)}\")\nend\n```","ref":"readme.html#automatic-cost-calculation"},{"type":"extras","title":"Token Estimation - ExLLM","doc":"```elixir\n# Estimate tokens before making a request\nmessages = [\n  %{role: \"system\", content: \"You are a helpful assistant.\"},\n  %{role: \"user\", content: \"Explain quantum computing in simple terms.\"}\n]\n\nestimated_tokens = ExLLM.estimate_tokens(messages)\n# Use this to predict costs before making the actual API call\n```","ref":"readme.html#token-estimation"},{"type":"extras","title":"Cost Comparison - ExLLM","doc":"```elixir\n# Compare costs across different providers\nusage = %{input_tokens: 1000, output_tokens: 2000}\n\nproviders = [\n  {:openai, \"gpt-4\"},\n  {:openai, \"gpt-3.5-turbo\"},\n  {:anthropic, \"claude-3-5-sonnet-20241022\"},\n  {:anthropic, \"claude-3-haiku-20240307\"}\n]\n\nEnum.each(providers, fn {provider, model} ->\n  cost = ExLLM.calculate_cost(provider, model, usage)\n  unless cost[:error] do\n    IO.puts(\"#{provider}/#{model}: #{ExLLM.format_cost(cost.total_cost)}\")\n  end\nend)\n```","ref":"readme.html#cost-comparison"},{"type":"extras","title":"Supported Pricing - ExLLM","doc":"ExLLM includes up-to-date pricing (as of January 2025) for:\n- OpenAI: GPT-4, GPT-4 Turbo, GPT-3.5 Turbo, GPT-4o series\n- Anthropic: Claude 3 series (Opus, Sonnet, Haiku), Claude 3.5, Claude 4\n- Google Gemini: Pro, Ultra, Nano\n- AWS Bedrock: Various models including Claude, Titan, Llama 2\n- Ollama: Local models (free - $0.00)","ref":"readme.html#supported-pricing"},{"type":"extras","title":"Context Management - ExLLM","doc":"ExLLM automatically manages context windows to ensure your messages fit within model limits:","ref":"readme.html#context-management"},{"type":"extras","title":"Automatic Context Truncation - ExLLM","doc":"```elixir\n# Long conversation that might exceed context window\nmessages = [\n  %{role: \"system\", content: \"You are a helpful assistant.\"},\n  # ... hundreds of messages ...\n  %{role: \"user\", content: \"What's my current task?\"}\n]\n\n# ExLLM automatically truncates to fit the model's context window\n{:ok, response} = ExLLM.chat(:anthropic, messages)\n```","ref":"readme.html#automatic-context-truncation"},{"type":"extras","title":"Context Window Validation - ExLLM","doc":"```elixir\n# Check if messages fit within context window\ncase ExLLM.validate_context(messages, model: \"gpt-3.5-turbo\") do\n  {:ok, token_count} ->\n    IO.puts(\"Messages use #{token_count} tokens\")\n  {:error, {:context_too_large, %{tokens: tokens, max_tokens: max}}} ->\n    IO.puts(\"Messages too large: #{tokens} tokens (max: #{max})\")\nend\n```","ref":"readme.html#context-window-validation"},{"type":"extras","title":"Context Strategies - ExLLM","doc":"```elixir\n# Sliding window (default) - keeps most recent messages\n{:ok, response} = ExLLM.chat(:anthropic, messages,\n  max_tokens: 4000,\n  strategy: :sliding_window\n)\n\n# Smart strategy - preserves system messages and recent context\n{:ok, response} = ExLLM.chat(:anthropic, messages,\n  max_tokens: 4000,\n  strategy: :smart,\n  preserve_messages: 10  # Always keep last 10 messages\n)\n```","ref":"readme.html#context-strategies"},{"type":"extras","title":"Context Statistics - ExLLM","doc":"```elixir\n# Get detailed statistics about your messages\nstats = ExLLM.context_stats(messages)\nIO.inspect(stats)\n# %{\n#   message_count: 150,\n#   total_tokens: 45000,\n#   by_role: %{\"system\" => 1, \"user\" => 75, \"assistant\" => 74},\n#   avg_tokens_per_message: 300\n# }\n\n# Check context window sizes\nIO.puts(ExLLM.context_window_size(:anthropic, \"claude-3-5-sonnet-20241022\"))\n# => 200000\n```","ref":"readme.html#context-statistics"},{"type":"extras","title":"Session Management - ExLLM","doc":"ExLLM includes built-in session management for maintaining conversation state:","ref":"readme.html#session-management-1"},{"type":"extras","title":"Creating and Using Sessions - ExLLM","doc":"```elixir\n# Create a new session\nsession = ExLLM.new_session(:anthropic, name: \"My Chat\")\n\n# Chat with automatic session tracking\n{:ok, {response, updated_session}} = ExLLM.chat_with_session(session, \"Hello!\")\n\n# Continue the conversation\n{:ok, {response2, session2}} = ExLLM.chat_with_session(updated_session, \"What's 2+2?\")\n\n# Access session messages\nmessages = ExLLM.get_session_messages(session2)\n# => [%{role: \"user\", content: \"Hello!\"}, %{role: \"assistant\", content: \"...\"}, ...]\n```","ref":"readme.html#creating-and-using-sessions"},{"type":"extras","title":"Session Persistence - ExLLM","doc":"```elixir\n# Save session to disk\n{:ok, path} = ExLLM.save_session(session, \"/path/to/sessions\")\n\n# Load session from disk\n{:ok, loaded_session} = ExLLM.load_session(\"/path/to/sessions/session_id.json\")\n\n# Export session as markdown\n{:ok, markdown} = ExLLM.export_session_markdown(session)\nFile.write!(\"conversation.md\", markdown)\n```","ref":"readme.html#session-persistence"},{"type":"extras","title":"Session Information - ExLLM","doc":"```elixir\n# Get session metadata\ninfo = ExLLM.session_info(session)\n# => %{\n#   id: \"123...\",\n#   name: \"My Chat\",\n#   created_at: ~U[2025-01-24 10:00:00Z],\n#   message_count: 10,\n#   total_tokens: 1500\n# }\n\n# Get token usage for session\ntokens = ExLLM.session_token_usage(session)\n# => 1500\n\n# Clear session messages\nclean_session = ExLLM.clear_session(session)\n```","ref":"readme.html#session-information"},{"type":"extras","title":"Structured Outputs - ExLLM","doc":"ExLLM integrates with [instructor_ex](https://github.com/thmsmlr/instructor_ex) to provide structured output validation. This allows you to define expected response structures using Ecto schemas and automatically validate LLM responses.","ref":"readme.html#structured-outputs"},{"type":"extras","title":"Installation - ExLLM","doc":"Add the optional instructor dependency:\n\n```elixir\ndef deps do\n  [\n    {:ex_llm, \"~> 0.1.0\"},\n    {:instructor, \"~> 0.1.0\"}  # Optional: for structured outputs\n  ]\nend\n```","ref":"readme.html#installation-1"},{"type":"extras","title":"Basic Usage - ExLLM","doc":"```elixir\n# Define your schema\ndefmodule EmailClassification do\n  use Ecto.Schema\n  use Instructor.Validator\n\n  @llm_doc \"Classification of an email as spam or not spam\"\n  \n  @primary_key false\n  embedded_schema do\n    field :classification, Ecto.Enum, values: [:spam, :not_spam]\n    field :confidence, :float\n    field :reason, :string\n  end\n\n  @impl true\n  def validate_changeset(changeset) do\n    changeset\n    |> Ecto.Changeset.validate_required([:classification, :confidence, :reason])\n    |> Ecto.Changeset.validate_number(:confidence, \n        greater_than_or_equal_to: 0.0,\n        less_than_or_equal_to: 1.0\n      )\n  end\nend\n\n# Use with ExLLM\nmessages = [%{role: \"user\", content: \"Is this spam? 'You won a million dollars!'\"}]\n\n{:ok, result} = ExLLM.chat(:anthropic, messages,\n  response_model: EmailClassification,\n  max_retries: 3  # Automatically retry on validation errors\n)\n\nIO.inspect(result)\n# %EmailClassification{\n#   classification: :spam,\n#   confidence: 0.95,\n#   reason: \"Classic lottery scam pattern\"\n# }\n```","ref":"readme.html#basic-usage-1"},{"type":"extras","title":"With Simple Type Specifications - ExLLM","doc":"```elixir\n# Define expected structure without Ecto\nresponse_model = %{\n  name: :string,\n  age: :integer,\n  email: :string,\n  tags: {:array, :string}\n}\n\nmessages = [%{role: \"user\", content: \"Extract: John Doe, 30 years old, john@example.com, likes elixir and coding\"}]\n\n{:ok, result} = ExLLM.chat(:anthropic, messages,\n  response_model: response_model\n)\n\nIO.inspect(result)\n# %{\n#   name: \"John Doe\",\n#   age: 30,\n#   email: \"john@example.com\",\n#   tags: [\"elixir\", \"coding\"]\n# }\n```","ref":"readme.html#with-simple-type-specifications"},{"type":"extras","title":"Advanced Example - ExLLM","doc":"```elixir\ndefmodule UserProfile do\n  use Ecto.Schema\n  use Instructor.Validator\n\n  @llm_doc \"\"\"\n  User profile extraction from text.\n  Extract all available information about the user.\n  \"\"\"\n\n  embedded_schema do\n    field :name, :string\n    field :email, :string\n    field :age, :integer\n    field :location, :string\n    embeds_many :interests, Interest do\n      field :name, :string\n      field :level, Ecto.Enum, values: [:beginner, :intermediate, :expert]\n    end\n  end\n\n  @impl true\n  def validate_changeset(changeset) do\n    changeset\n    |> Ecto.Changeset.validate_required([:name])\n    |> Ecto.Changeset.validate_format(:email, ~r/@/)\n    |> Ecto.Changeset.validate_number(:age, greater_than: 0, less_than: 150)\n  end\nend\n\n# Complex extraction with nested structures\ntext = \"\"\"\nHi, I'm Jane Smith, a 28-year-old software engineer from Seattle.\nYou can reach me at jane.smith@tech.com. I'm an expert in Elixir,\nintermediate in Python, and just starting to learn Rust.\n\"\"\"\n\n{:ok, profile} = ExLLM.chat(:anthropic, \n  [%{role: \"user\", content: \"Extract user profile: #{text}\"}],\n  response_model: UserProfile,\n  max_retries: 3\n)\n```","ref":"readme.html#advanced-example"},{"type":"extras","title":"Using the Instructor Module Directly - ExLLM","doc":"```elixir\n# Direct usage of ExLLM.Instructor\n{:ok, result} = ExLLM.Instructor.chat(:anthropic, messages,\n  response_model: EmailClassification,\n  max_retries: 3,\n  temperature: 0.1  # Lower temperature for more consistent structure\n)\n\n# Parse an existing response\n{:ok, response} = ExLLM.chat(:anthropic, messages)\n{:ok, structured} = ExLLM.Instructor.parse_response(response, UserProfile)\n\n# Check if instructor is available\nif ExLLM.Instructor.available?() do\n  # Use structured outputs\nelse\n  # Fall back to regular parsing\nend\n```","ref":"readme.html#using-the-instructor-module-directly"},{"type":"extras","title":"Supported Providers - ExLLM","doc":"Structured outputs work with providers that have instructor adapters:\n- `:anthropic` - Anthropic Claude\n- `:openai` - OpenAI GPT models (coming soon)\n- `:ollama` - Local Ollama models (coming soon)\n- `:gemini` - Google Gemini (coming soon)","ref":"readme.html#supported-providers-1"},{"type":"extras","title":"Error Handling - ExLLM","doc":"```elixir\ncase ExLLM.chat(:anthropic, messages, response_model: UserProfile) do\n  {:ok, profile} ->\n    # Successfully validated structure\n    IO.inspect(profile)\n    \n  {:error, :instructor_not_available} ->\n    # Instructor library not installed\n    IO.puts(\"Please install instructor to use structured outputs\")\n    \n  {:error, {:validation_failed, errors}} ->\n    # Validation failed after retries\n    IO.inspect(errors)\n    \n  {:error, reason} ->\n    # Other error\n    IO.inspect(reason)\nend\n```","ref":"readme.html#error-handling"},{"type":"extras","title":"Configuration - ExLLM","doc":"ExLLM supports multiple configuration providers:","ref":"readme.html#configuration-1"},{"type":"extras","title":"Environment Variables (Default) - ExLLM","doc":"```elixir\n# Uses ExLLM.ConfigProvider.Default\n# Reads from application config and environment variables\n```","ref":"readme.html#environment-variables-default"},{"type":"extras","title":"Static Configuration - ExLLM","doc":"```elixir\nconfig = %{\n  anthropic: [\n    api_key: \"your-api-key\",\n    base_url: \"https://api.anthropic.com\"\n  ]\n}\n\nExLLM.set_config_provider({ExLLM.ConfigProvider.Static, config})\n```","ref":"readme.html#static-configuration"},{"type":"extras","title":"Custom Configuration Provider - ExLLM","doc":"```elixir\ndefmodule MyConfigProvider do\n  @behaviour ExLLM.ConfigProvider\n\n  @impl true\n  def get_config(provider, key) do\n    # Your custom logic here\n  end\n\n  @impl true\n  def has_config?(provider) do\n    # Your custom logic here\n  end\nend\n\nExLLM.set_config_provider(MyConfigProvider)\n```","ref":"readme.html#custom-configuration-provider"},{"type":"extras","title":"Error Handling - ExLLM","doc":"ExLLM uses consistent error patterns:\n\n```elixir\ncase ExLLM.chat(:anthropic, messages) do\n  {:ok, response} ->\n    # Success\n    IO.puts(response.content)\n\n  {:error, {:config_error, reason}} ->\n    # Configuration issue\n    IO.puts(\"Config error: #{reason}\")\n\n  {:error, {:api_error, %{status: status, body: body}}} ->\n    # API error\n    IO.puts(\"API error #{status}: #{body}\")\n\n  {:error, {:network_error, reason}} ->\n    # Network issue\n    IO.puts(\"Network error: #{reason}\")\n\n  {:error, {:parse_error, reason}} ->\n    # Response parsing issue\n    IO.puts(\"Parse error: #{reason}\")\nend\n```","ref":"readme.html#error-handling-1"},{"type":"extras","title":"Local Model Support - ExLLM","doc":"ExLLM supports running models locally using Bumblebee and EXLA/EMLX backends. This enables on-device inference without API calls or costs.","ref":"readme.html#local-model-support"},{"type":"extras","title":"Setup - ExLLM","doc":"1. Add optional dependencies to your `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_llm, \"~> 0.1.0\"},\n    {:bumblebee, \"~> 0.5\"},\n    {:nx, \"~> 0.7\"},\n    {:exla, \"~> 0.7\"}  # or {:emlx, \"~> 0.1\"} for Apple Silicon\n  ]\nend\n```\n\n2. Configure EXLA backend (optional - auto-detected by default):\n\n```elixir\n# For CUDA GPUs\nconfig :nx, :default_backend, {EXLA.Backend, client: :cuda}\n\n# For Apple Silicon\nconfig :nx, :default_backend, EMLX.Backend\n```","ref":"readme.html#setup"},{"type":"extras","title":"Available Models - ExLLM","doc":"- **microsoft/phi-2** - Phi-2 (2.7B parameters) - Default\n- **meta-llama/Llama-2-7b-hf** - Llama 2 (7B)\n- **mistralai/Mistral-7B-v0.1** - Mistral (7B)\n- **EleutherAI/gpt-neo-1.3B** - GPT-Neo (1.3B)\n- **google/flan-t5-base** - Flan-T5 Base","ref":"readme.html#available-models"},{"type":"extras","title":"Usage - ExLLM","doc":"```elixir\n# Start the model loader (happens automatically on first use)\n{:ok, _} = ExLLM.Local.ModelLoader.start_link()\n\n# Use a local model\nmessages = [\n  %{role: \"user\", content: \"Explain quantum computing in simple terms\"}\n]\n\n{:ok, response} = ExLLM.chat(:local, messages, model: \"microsoft/phi-2\")\nIO.puts(response.content)\n\n# Stream responses\n{:ok, stream} = ExLLM.stream_chat(:local, messages)\nfor chunk <- stream do\n  IO.write(chunk.content)\nend\n\n# List available models\n{:ok, models} = ExLLM.list_models(:local)\nEnum.each(models, fn model ->\n  IO.puts(\"#{model.name} - Context: #{model.context_window} tokens\")\nend)\n\n# Check acceleration info\ninfo = ExLLM.Local.EXLAConfig.acceleration_info()\nIO.puts(\"Running on: #{info.name}\")\n```","ref":"readme.html#usage"},{"type":"extras","title":"Hardware Acceleration - ExLLM","doc":"ExLLM automatically detects and uses available hardware acceleration:\n\n- **Apple Silicon** - Uses Metal via EMLX\n- **NVIDIA GPUs** - Uses CUDA via EXLA\n- **AMD GPUs** - Uses ROCm via EXLA\n- **CPUs** - Optimized multi-threaded inference","ref":"readme.html#hardware-acceleration"},{"type":"extras","title":"Performance Tips - ExLLM","doc":"1. **First Load**: Models are downloaded from HuggingFace on first use and cached locally\n2. **Memory**: Ensure you have enough RAM/VRAM for your chosen model\n3. **Batch Size**: Automatically optimized based on available memory\n4. **Mixed Precision**: Enabled by default for better performance","ref":"readme.html#performance-tips"},{"type":"extras","title":"Model Loading - ExLLM","doc":"```elixir\n# Pre-load a model\n{:ok, _} = ExLLM.Local.ModelLoader.load_model(\"microsoft/phi-2\")\n\n# Load from local path\n{:ok, _} = ExLLM.Local.ModelLoader.load_model(\"/path/to/model\")\n\n# Unload to free memory\n:ok = ExLLM.Local.ModelLoader.unload_model(\"microsoft/phi-2\")\n\n# List loaded models\nloaded = ExLLM.Local.ModelLoader.list_loaded_models()\n```","ref":"readme.html#model-loading"},{"type":"extras","title":"Adding New Providers - ExLLM","doc":"To add a new LLM provider, implement the `ExLLM.Adapter` behaviour:\n\n```elixir\ndefmodule ExLLM.Adapters.MyProvider do\n  @behaviour ExLLM.Adapter\n\n  @impl true\n  def chat(messages, options) do\n    # Implement chat completion\n  end\n\n  @impl true\n  def stream_chat(messages, options, callback) do\n    # Implement streaming chat\n  end\n\n  @impl true\n  def configured?() do\n    # Check if provider is configured\n  end\n\n  @impl true\n  def list_models() do\n    # Return available models\n  end\nend\n```\n\nThen register it in the main ExLLM module.","ref":"readme.html#adding-new-providers"},{"type":"extras","title":"Testing - ExLLM","doc":"Run the test suite:\n\n```bash\nmix test\n```","ref":"readme.html#testing"},{"type":"extras","title":"Contributing - ExLLM","doc":"1. Fork the repository\n2. Create a feature branch\n3. Make your changes\n4. Add tests\n5. Run `mix format` and `mix credo`\n6. Submit a pull request","ref":"readme.html#contributing"},{"type":"extras","title":"License - ExLLM","doc":"MIT License - see [LICENSE](LICENSE) for details.","ref":"readme.html#license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}